@@ -5,7 +5,7 @@ except ImportError:
     from total_ordering import total_ordering
 
 
-__version__ = '1.1'
+__version__ = '1.2'
 
 
 @total_ordering
@@ -45,21 +45,21 @@ class Infinity(object):
         return float(str(self))
 
     def __add__(self, other):
-        if (
-            isinstance(other, self.__class__) and
-            other.positive != self.positive
-        ):
+        if is_infinite(other) and other != self:
             return NotImplemented
         return self
 
+    def __radd__(self, other):
+        return self
+
     def __sub__(self, other):
-        if (
-            isinstance(other, self.__class__) and
-            other.positive == self.positive
-        ):
+        if is_infinite(other) and other == self:
             return NotImplemented
         return self
 
+    def __rsub__(self, other):
+        return self
+
     def timetuple(self):
         return tuple()
 
@@ -70,14 +70,23 @@ class Infinity(object):
         return self
 
     def __div__(self, other):
-        if isinstance(other, self.__class__):
+        if is_infinite(other):
             return NotImplemented
 
         return Infinity(
             other > 0 and self.positive or other < 0 and not self.positive
         )
 
+    def __rdiv__(self, other):
+        return 0
+
+    def __repr__(self):
+        return str(self)
+
     __truediv__ = __div__
+    __rtruediv__ = __rdiv__
+    __floordiv__ = __div__
+    __rfloordiv__ = __rdiv__
 
     def __mul__(self, other):
         if other is 0:
@@ -86,6 +95,8 @@ class Infinity(object):
             other > 0 and self.positive or other < 0 and not self.positive
         )
 
+    __rmul__ = __mul__
+
     def __pow__(self, other):
         if other is 0:
             return NotImplemented
@@ -104,3 +115,7 @@ class Infinity(object):
 
 
 inf = Infinity()
+
+
+def is_infinite(value):
+    return value == inf or value == -inf
